/*-
 * Copyright (c) 2006 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include <sys/types.h>
#include <sys/cdefs.h>
#include <errno.h>

#define	SYM_COMPAT(sym, impl, ver)		\
	__sym_compat(sym, impl, ver)

#define	GLIBC_SYMBOL_2(sym, major, minor)	\
	GLIBC_##major##_##minor##_##sym

#define	GLIBC_SYMBOL_3(sym, major, minor, patch)	\
	GLIBC_##major##_##minor##_##patch##_##sym

#define	GLIBC_COMPAT_2(sym, major, minor)	\
	SYM_COMPAT(sym, GLIBC_SYMBOL_2(sym, major, minor), GLIBC_ ##major. ##minor);

#define	GLIBC_COMPAT_3(sym, major, minor, patch)	\
	SYM_COMPAT(sym, GLIBC_SYMBOL_3(sym, major, minor, patch), GLIBC_ ##major. ##minor. ##patch);

#ifdef  DEBUG
#	include <stdio.h>
#	include <unistd.h>
#	define	dprintf(str, ...)	if(  getenv("UCLT")  )  fprintf(stderr, "[%d] " str "\n", getpid() ,##__VA_ARGS__)
#	define	CLEAR_ERRNO		errno = 0
#else
#	define	dprintf(str, ...)	/* NOTHING TO DO */
#	define	CLEAR_ERRNO		/* NOTHING TO DO */
#endif

/* flags of open(2) */
#define LINUX_O_RDONLY			00
#define LINUX_O_WRONLY			01
#define LINUX_O_RDWR			02
#define LINUX_O_NDELAY			LINUX_O_NONBLOCK
#define LINUX_O_APPEND			02000
#define LINUX_O_SYNC			010000
#define LINUX_O_NONBLOCK		04000
#define LINUX_FASYNC			020000
#define LINUX_O_CREAT			0100
#define LINUX_O_TRUNC			01000
#define LINUX_O_EXCL			0200
#define LINUX_O_NOCTTY			0400

/* requests of ioctl(2) */
#if defined(__i386__) || defined(__amd64__)
#define _LINUX_IOC_NRBITS		8
#define _LINUX_IOC_TYPEBITS		8
#define _LINUX_IOC_SIZEBITS		14
#define _LINUX_IOC_DIRBITS		2
#else
#error	NOT SUPPORT ARCHITECTURE
#endif

#define _LINUX_IOC_NRMASK		( (1 << _LINUX_IOC_NRBITS  ) - 1 )
#define _LINUX_IOC_TYPEMASK		( (1 << _LINUX_IOC_TYPEBITS) - 1 )
#define _LINUX_IOC_SIZEMASK		( (1 << _LINUX_IOC_SIZEBITS) - 1 )
#define _LINUX_IOC_DIRMASK		( (1 << _LINUX_IOC_DIRBITS ) - 1 )

#define _LINUX_IOC_NRSHIFT		0
#define _LINUX_IOC_TYPESHIFT		(_LINUX_IOC_NRSHIFT   + _LINUX_IOC_NRBITS)
#define _LINUX_IOC_SIZESHIFT		(_LINUX_IOC_TYPESHIFT + _LINUX_IOC_TYPEBITS)
#define _LINUX_IOC_DIRSHIFT		(_LINUX_IOC_SIZESHIFT + _LINUX_IOC_SIZEBITS)

#define _LINUX_IOC_NONE			0U
#define _LINUX_IOC_WRITE		1U
#define _LINUX_IOC_READ			2U

#define _LINUX_IOC(dir, type, nr, size)		\
	(  ( (dir)  << _LINUX_IOC_DIRSHIFT  ) | \
	   ( (type) << _LINUX_IOC_TYPESHIFT ) | \
	   ( (nr)   << _LINUX_IOC_NRSHIFT   ) | \
	   ( (size) << _LINUX_IOC_SIZESHIFT )  )

#define _LINUX_IO(type, nr)			\
		_LINUX_IOC(_LINUX_IOC_NONE,		     (type), (nr), 0)
#define _LINUX_IOR(type, nr, size)		\
		_LINUX_IOC(_LINUX_IOC_READ,		     (type), (nr), sizeof(size))
#define _LINUX_IOW(type, nr, size)		\
		_LINUX_IOC(_LINUX_IOC_WRITE,		     (type), (nr), sizeof(size))
#define _LINUX_IOWR(type, nr, size)		\
		_LINUX_IOC(_LINUX_IOC_READ|_LINUX_IOC_WRITE, (type), (nr), sizeof(size))

/* sound device of ioctl(2) requests */
#define LINUX_SNDCTL_DSP_RESET                  _LINUX_IO  ('P',   0)
#define LINUX_SNDCTL_DSP_SYNC                   _LINUX_IO  ('P',   1)
#define LINUX_SNDCTL_DSP_SETFMT                 _LINUX_IOWR('P',   5, int)
#define LINUX_SNDCTL_DSP_STEREO                 _LINUX_IOWR('P',   3, int)
#define LINUX_SNDCTL_DSP_SPEED                  _LINUX_IOWR('P',   2, int)
#define LINUX_SNDCTL_DSP_CHANNELS               _LINUX_IOWR('P',   6, int)
#define LINUX_SNDCTL_DSP_SETFRAGMENT            _LINUX_IOWR('P',  10, int)
#define LINUX_SNDCTL_DSP_GETFMTS                _LINUX_IOR ('P',  11, int)
#define LINUX_SNDCTL_DSP_GETCAPS                _LINUX_IOR ('P',  15, int)
#define LINUX_SNDCTL_DSP_GETOPTR                _LINUX_IOR ('P',  18, linux_count_info)
#define LINUX_SNDCTL_DSP_GETOSPACE              _LINUX_IOR ('P',  12, linux_audio_buf_info)
#define LINUX_MIXER_READ(dev)                   _LINUX_IOR ('M', dev, int)
#define LINUX_SOUND_MIXER_DEVMASK               0xfe
#define LINUX_SOUND_MIXER_READ_DEVMASK          LINUX_MIXER_READ(LINUX_SOUND_MIXER_DEVMASK)
#define LINUX_SOUND_PCM_READ_CHANNELS           _LINUX_IOR ('P',   6, int)
#define LINUX_SOUND_PCM_WRITE_CHANNELS          LINUX_SNDCTL_DSP_CHANNELS

#define	LINUX_FIONREAD				_LINUX_IOW ('f', 127, int)


/* commands of fcntl(2) */
#define LINUX_F_DUPFD                   0
#define LINUX_F_GETFD                   1
#define LINUX_F_SETFD                   2
#define LINUX_F_GETFL                   3
#define LINUX_F_SETFL                   4
#define LINUX_F_GETLK                   5
#define LINUX_F_SETLK                   6
#define LINUX_F_SETLKW                  7
#define LINUX_F_SETOWN                  8
#define LINUX_F_GETOWN                  9

/* flags of shmget(2) */
#define	LINUX_SHM_R			0400
#define	LINUX_SHM_W			0200

/* flags of shmat(2) */
#define	LINUX_SHM_RDONLY		010000
#define	LINUX_SHM_RND			020000
#define	LINUX_SHM_REMAP			040000

/* commands of shmctl */
#define	LINUX_SHM_LOCK			11
#define	LINUX_SHM_UNLOCK		12

/* sizeof struct utsname */
#define	UTSNAME_LENGTH			65

/* errno */
#define	LINUX_EAGAIN			11

/* signal */
#define	LINUX_SIGHUP			1
#define	LINUX_SIGINT			2
#define	LINUX_SIGQUIT			3
#define	LINUX_SIGILL			4
#define	LINUX_SIGTRAP			5
#define	LINUX_SIGABRT			6
#define	LINUX_SIGIOT			LINUX_SIGABRT
#define	LINUX_SIGBUS			7
#define	LINUX_SIGFPE			8
#define	LINUX_SIGKILL			9
#define	LINUX_SIGUSR1			10
#define	LINUX_SIGSEGV			11
#define	LINUX_SIGUSR2			12
#define	LINUX_SIGPIPE			13
#define	LINUX_SIGALRM			14
#define	LINUX_SIGTERM			15
#define	LINUX_SIGSTKFLT			16
#define	LINUX_SIGCHLD			17
#define	LINUX_SIGCONT			18
#define	LINUX_SIGSTOP			19
#define	LINUX_SIGTSTP			20
#define	LINUX_SIGTTIN			21
#define	LINUX_SIGTTOU			22
#define	LINUX_SIGURG			23
#define	LINUX_SIGXCPU			24
#define	LINUX_SIGXFSZ			25
#define	LINUX_SIGVTALRM			26
#define	LINUX_SIGPROF			27
#define	LINUX_SIGWINCH			28
#define	LINUX_SIGIO			29
#define	LINUX_SIGPOLL			LINUX_SIGIO
#define	LINUX_SIGPWR			30
#define	LINUX_SIGSYS			31
#define	LINUX_SIGTBLSZ			31
#define	LINUX_NSIG			32
#define	LINUX_NSIG_WORDS		2

#define	LINUX_SA_NOCLDSTOP		0x00000001
#define	LINUX_SA_NOCLDWAIT		0x00000002
#define	LINUX_SA_SIGINFO		0x00000004
#define	LINUX_SA_RESTORER		0x04000000
#define	LINUX_SA_ONSTACK		0x08000000
#define	LINUX_SA_RESTART		0x10000000
#define	LINUX_SA_INTERRUPT		0x20000000
#define	LINUX_SA_NOMASK			0x40000000
#define	LINUX_SA_ONESHOT		0x80000000

/* socket */
#define LINUX_AF_UNSPEC			0
#define LINUX_AF_UNIX			1
#define LINUX_AF_INET			2
#define LINUX_AF_AX25			3
#define LINUX_AF_IPX			4
#define LINUX_AF_APPLETALK		5
#define LINUX_AF_INET6			10

/* sockopt */
#define LINUX_SO_DEBUG			1
#define LINUX_SO_REUSEADDR		2
#define LINUX_SO_TYPE			3
#define LINUX_SO_ERROR			4
#define LINUX_SO_DONTROUTE		5
#define LINUX_SO_BROADCAST		6
#define LINUX_SO_SNDBUF			7
#define LINUX_SO_RCVBUF			8
#define LINUX_SO_KEEPALIVE		9
#define LINUX_SO_OOBINLINE		10
#define LINUX_SO_NO_CHECK		11
#define LINUX_SO_PRIORITY		12
#define LINUX_SO_LINGER			13

#define	LINUX_IP_TOS			1
#define	LINUX_IP_TTL			2
#define	LINUX_IP_HDRINCL		3
#define	LINUX_IP_OPTIONS		4

#define	LINUX_SOL_SOCKET		1
#define	LINUX_SOL_IP			0
#define	LINUX_SOL_IPX			256
#define	LINUX_SOL_AX25			257
#define	LINUX_SOL_TCP			6
#define	LINUX_SOL_UDP			17

/* sysconf */
enum {
	LINUX_SC_ARG_MAX_SC_ARG_MAX,
	LINUX_SC_CHILD_MAX_SC_CHILD_MAX,
	LINUX_SC_CLK_TCK_SC_CLK_TCK,
	LINUX_SC_NGROUPS_MAX_SC_NGROUPS_MAX,
	LINUX_SC_OPEN_MAX_SC_OPEN_MAX,
	LINUX_SC_STREAM_MAX_SC_STREAM_MAX,
	LINUX_SC_TZNAME_MAX_SC_TZNAME_MAX,
	LINUX_SC_JOB_CONTROL_SC_JOB_CONTROL,
	LINUX_SC_SAVED_IDS_SC_SAVED_IDS,
	LINUX_SC_REALTIME_SIGNALS_SC_REALTIME_SIGNALS,
	LINUX_SC_PRIORITY_SCHEDULING_SC_PRIORITY_SCHEDULING,
	LINUX_SC_TIMERS_SC_TIMERS,
	LINUX_SC_ASYNCHRONOUS_IO_SC_ASYNCHRONOUS_IO,
	LINUX_SC_PRIORITIZED_IO_SC_PRIORITIZED_IO,
	LINUX_SC_SYNCHRONIZED_IO_SC_SYNCHRONIZED_IO,
	LINUX_SC_FSYNC_SC_FSYNC,
	LINUX_SC_MAPPED_FILES_SC_MAPPED_FILES,
	LINUX_SC_MEMLOCK_SC_MEMLOCK,
	LINUX_SC_MEMLOCK_RANGE_SC_MEMLOCK_RANGE,
	LINUX_SC_MEMORY_PROTECTION_SC_MEMORY_PROTECTION,
	LINUX_SC_MESSAGE_PASSING_SC_MESSAGE_PASSING,
	LINUX_SC_SEMAPHORES_SC_SEMAPHORES,
	LINUX_SC_SHARED_MEMORY_OBJECTS_SC_SHARED_MEMORY_OBJECTS,
	LINUX_SC_AIO_LISTIO_MAX_SC_AIO_LISTIO_MAX,
	LINUX_SC_AIO_MAX_SC_AIO_MAX,
	LINUX_SC_AIO_PRIO_DELTA_MAX_SC_AIO_PRIO_DELTA_MAX,
	LINUX_SC_DELAYTIMER_MAX_SC_DELAYTIMER_MAX,
	LINUX_SC_MQ_OPEN_MAX_SC_MQ_OPEN_MAX,
	LINUX_SC_MQ_PRIO_MAX_SC_MQ_PRIO_MAX,
	LINUX_SC_VERSION_SC_VERSION,
	LINUX_SC_PAGESIZE_SC_PAGESIZE,
	LINUX_SC_PAGE_SIZE_SC_PAGESIZE,
	LINUX_SC_RTSIG_MAX_SC_RTSIG_MAX,
	LINUX_SC_SEM_NSEMS_MAX_SC_SEM_NSEMS_MAX,
	LINUX_SC_SEM_VALUE_MAX_SC_SEM_VALUE_MAX,
	LINUX_SC_SIGQUEUE_MAX_SC_SIGQUEUE_MAX,
	LINUX_SC_TIMER_MAX_SC_TIMER_MAX,
	LINUX_SC_BC_BASE_MAX_SC_BC_BASE_MAX,
	LINUX_SC_BC_DIM_MAX_SC_BC_DIM_MAX,
	LINUX_SC_BC_SCALE_MAX_SC_BC_SCALE_MAX,
	LINUX_SC_BC_STRING_MAX_SC_BC_STRING_MAX,
	LINUX_SC_COLL_WEIGHTS_MAX_SC_COLL_WEIGHTS_MAX,
	LINUX_SC_EQUIV_CLASS_MAX_SC_EQUIV_CLASS_MAX,
	LINUX_SC_EXPR_NEST_MAX_SC_EXPR_NEST_MAX,
	LINUX_SC_LINE_MAX_SC_LINE_MAX,
	LINUX_SC_RE_DUP_MAX_SC_RE_DUP_MAX,
	LINUX_SC_CHARCLASS_NAME_MAX_SC_CHARCLASS_NAME_MAX,
	LINUX_SC_2_VERSION_SC_2_VERSION,
	LINUX_SC_2_C_BIND_SC_2_C_BIND,
	LINUX_SC_2_C_DEV_SC_2_C_DEV,
	LINUX_SC_2_FORT_DEV_SC_2_FORT_DEV,
	LINUX_SC_2_FORT_RUN_SC_2_FORT_RUN,
	LINUX_SC_2_SW_DEV_SC_2_SW_DEV,
	LINUX_SC_2_LOCALEDEF_SC_2_LOCALEDEF,
	LINUX_SC_PII_SC_PII,
	LINUX_SC_PII_XTI_SC_PII_XTI,
	LINUX_SC_PII_SOCKET_SC_PII_SOCKET,
	LINUX_SC_PII_INTERNET_SC_PII_INTERNET,
	LINUX_SC_PII_OSI_SC_PII_OSI,
	LINUX_SC_POLL_SC_POLL,
	LINUX_SC_SELECT_SC_SELECT,
	LINUX_SC_UIO_MAXIOV_SC_UIO_MAXIOV,
	LINUX_SC_IOV_MAX_SC_IOV_MAX,
	LINUX_SC_PII_INTERNET_STREAM_SC_PII_INTERNET_STREAM,
	LINUX_SC_PII_INTERNET_DGRAM_SC_PII_INTERNET_DGRAM,
	LINUX_SC_PII_OSI_COTS_SC_PII_OSI_COTS,
	LINUX_SC_PII_OSI_CLTS_SC_PII_OSI_CLTS,
	LINUX_SC_PII_OSI_M_SC_PII_OSI_M,
	LINUX_SC_T_IOV_MAX_SC_T_IOV_MAX,
	LINUX_SC_THREADS_SC_THREADS,
	LINUX_SC_THREAD_SAFE_FUNCTIONS_SC_THREAD_SAFE_FUNCTIONS,
	LINUX_SC_GETGR_R_SIZE_MAX_SC_GETGR_R_SIZE_MAX,
	LINUX_SC_GETPW_R_SIZE_MAX_SC_GETPW_R_SIZE_MAX,
	LINUX_SC_LOGIN_NAME_MAX_SC_LOGIN_NAME_MAX,
	LINUX_SC_TTY_NAME_MAX_SC_TTY_NAME_MAX,
	LINUX_SC_THREAD_DESTRUCTOR_ITERATIONS_SC_THREAD_DESTRUCTOR_ITERATIONS,
	LINUX_SC_THREAD_KEYS_MAX_SC_THREAD_KEYS_MAX,
	LINUX_SC_THREAD_STACK_MIN_SC_THREAD_STACK_MIN,
	LINUX_SC_THREAD_THREADS_MAX_SC_THREAD_THREADS_MAX,
	LINUX_SC_THREAD_ATTR_STACKADDR_SC_THREAD_ATTR_STACKADDR,
	LINUX_SC_THREAD_ATTR_STACKSIZE_SC_THREAD_ATTR_STACKSIZE,
	LINUX_SC_THREAD_PRIORITY_SCHEDULING_SC_THREAD_PRIORITY_SCHEDULING,
	LINUX_SC_THREAD_PRIO_INHERIT_SC_THREAD_PRIO_INHERIT,
	LINUX_SC_THREAD_PRIO_PROTECT_SC_THREAD_PRIO_PROTECT,
	LINUX_SC_THREAD_PROCESS_SHARED_SC_THREAD_PROCESS_SHARED,
	LINUX_SC_NPROCESSORS_CONF_SC_NPROCESSORS_CONF,
	LINUX_SC_NPROCESSORS_ONLN_SC_NPROCESSORS_ONLN,
	LINUX_SC_PHYS_PAGES_SC_PHYS_PAGES,
	LINUX_SC_AVPHYS_PAGES_SC_AVPHYS_PAGES,
	LINUX_SC_ATEXIT_MAX_SC_ATEXIT_MAX,
	LINUX_SC_PASS_MAX_SC_PASS_MAX,
	LINUX_SC_XOPEN_VERSION_SC_XOPEN_VERSION,
	LINUX_SC_XOPEN_XCU_VERSION_SC_XOPEN_XCU_VERSION,
	LINUX_SC_XOPEN_UNIX_SC_XOPEN_UNIX,
	LINUX_SC_XOPEN_CRYPT_SC_XOPEN_CRYPT,
	LINUX_SC_XOPEN_ENH_I18N_SC_XOPEN_ENH_I18N,
	LINUX_SC_XOPEN_SHM_SC_XOPEN_SHM,
	LINUX_SC_2_CHAR_TERM_SC_2_CHAR_TERM,
	LINUX_SC_2_C_VERSION_SC_2_C_VERSION,
	LINUX_SC_2_UPE_SC_2_UPE,
	LINUX_SC_XOPEN_XPG2_SC_XOPEN_XPG2,
	LINUX_SC_XOPEN_XPG3_SC_XOPEN_XPG3,
	LINUX_SC_XOPEN_XPG4_SC_XOPEN_XPG4,
	LINUX_SC_CHAR_BIT_SC_CHAR_BIT,
	LINUX_SC_CHAR_MAX_SC_CHAR_MAX,
	LINUX_SC_CHAR_MIN_SC_CHAR_MIN,
	LINUX_SC_INT_MAX_SC_INT_MAX,
	LINUX_SC_INT_MIN_SC_INT_MIN,
	LINUX_SC_LONG_BIT_SC_LONG_BIT,
	LINUX_SC_WORD_BIT_SC_WORD_BIT,
	LINUX_SC_MB_LEN_MAX_SC_MB_LEN_MAX,
	LINUX_SC_NZERO_SC_NZERO,
	LINUX_SC_SSIZE_MAX_SC_SSIZE_MAX,
	LINUX_SC_SCHAR_MAX_SC_SCHAR_MAX,
	LINUX_SC_SCHAR_MIN_SC_SCHAR_MIN,
	LINUX_SC_SHRT_MAX_SC_SHRT_MAX,
	LINUX_SC_SHRT_MIN_SC_SHRT_MIN,
	LINUX_SC_UCHAR_MAX_SC_UCHAR_MAX,
	LINUX_SC_UINT_MAX_SC_UINT_MAX,
	LINUX_SC_ULONG_MAX_SC_ULONG_MAX,
	LINUX_SC_USHRT_MAX_SC_USHRT_MAX,
	LINUX_SC_NL_ARGMAX_SC_NL_ARGMAX,
	LINUX_SC_NL_LANGMAX_SC_NL_LANGMAX,
	LINUX_SC_NL_MSGMAX_SC_NL_MSGMAX,
	LINUX_SC_NL_NMAX_SC_NL_NMAX,
	LINUX_SC_NL_SETMAX_SC_NL_SETMAX,
	LINUX_SC_NL_TEXTMAX_SC_NL_TEXTMAX,
	LINUX_SC_XBS5_ILP32_OFF32_SC_XBS5_ILP32_OFF32,
	LINUX_SC_XBS5_ILP32_OFFBIG_SC_XBS5_ILP32_OFFBIG,
	LINUX_SC_XBS5_LP64_OFF64_SC_XBS5_LP64_OFF64,
	LINUX_SC_XBS5_LPBIG_OFFBIG_SC_XBS5_LPBIG_OFFBIG,
	LINUX_SC_XOPEN_LEGACY_SC_XOPEN_LEGACY,
	LINUX_SC_XOPEN_REALTIME_SC_XOPEN_REALTIME,
	LINUX_SC_XOPEN_REALTIME_THREADS_SC_XOPEN_REALTIME_THREADS,
	LINUX_SC_ADVISORY_INFO_SC_ADVISORY_INFO,
	LINUX_SC_BARRIERS_SC_BARRIERS,
	LINUX_SC_BASE_SC_BASE,
	LINUX_SC_C_LANG_SUPPORT_SC_C_LANG_SUPPORT,
	LINUX_SC_C_LANG_SUPPORT_R_SC_C_LANG_SUPPORT_R,
	LINUX_SC_CLOCK_SELECTION_SC_CLOCK_SELECTION,
	LINUX_SC_CPUTIME_SC_CPUTIME,
	LINUX_SC_THREAD_CPUTIME_SC_THREAD_CPUTIME,
	LINUX_SC_DEVICE_IO_SC_DEVICE_IO,
	LINUX_SC_DEVICE_SPECIFIC_SC_DEVICE_SPECIFIC,
	LINUX_SC_DEVICE_SPECIFIC_R_SC_DEVICE_SPECIFIC_R,
	LINUX_SC_FD_MGMT_SC_FD_MGMT,
	LINUX_SC_FIFO_SC_FIFO,
	LINUX_SC_PIPE_SC_PIPE,
	LINUX_SC_FILE_ATTRIBUTES_SC_FILE_ATTRIBUTES,
	LINUX_SC_FILE_LOCKING_SC_FILE_LOCKING,
	LINUX_SC_FILE_SYSTEM_SC_FILE_SYSTEM,
	LINUX_SC_MONOTONIC_CLOCK_SC_MONOTONIC_CLOCK,
	LINUX_SC_MULTI_PROCESS_SC_MULTI_PROCESS,
	LINUX_SC_SINGLE_PROCESS_SC_SINGLE_PROCESS,
	LINUX_SC_NETWORKING_SC_NETWORKING,
	LINUX_SC_READER_WRITER_LOCKS_SC_READER_WRITER_LOCKS,
	LINUX_SC_SPIN_LOCKS_SC_SPIN_LOCKS,
	LINUX_SC_REGEXP_SC_REGEXP,
	LINUX_SC_REGEX_VERSION_SC_REGEX_VERSION,
	LINUX_SC_SHELL_SC_SHELL,
	LINUX_SC_SIGNALS_SC_SIGNALS,
	LINUX_SC_SPAWN_SC_SPAWN,
	LINUX_SC_SPORADIC_SERVER_SC_SPORADIC_SERVER,
	LINUX_SC_THREAD_SPORADIC_SERVER_SC_THREAD_SPORADIC_SERVER,
	LINUX_SC_SYSTEM_DATABASE_SC_SYSTEM_DATABASE,
	LINUX_SC_SYSTEM_DATABASE_R_SC_SYSTEM_DATABASE_R,
	LINUX_SC_TIMEOUTS_SC_TIMEOUTS,
	LINUX_SC_TYPED_MEMORY_OBJECTS_SC_TYPED_MEMORY_OBJECTS,
	LINUX_SC_USER_GROUPS_SC_USER_GROUPS,
	LINUX_SC_USER_GROUPS_R_SC_USER_GROUPS_R,
	LINUX_SC_2_PBS_SC_2_PBS,
	LINUX_SC_2_PBS_ACCOUNTING_SC_2_PBS_ACCOUNTING,
	LINUX_SC_2_PBS_LOCATE_SC_2_PBS_LOCATE,
	LINUX_SC_2_PBS_MESSAGE_SC_2_PBS_MESSAGE,
	LINUX_SC_2_PBS_TRACK_SC_2_PBS_TRACK,
	LINUX_SC_SYMLOOP_MAX_SC_SYMLOOP_MAX,
	LINUX_SC_STREAMS_SC_STREAMS,
	LINUX_SC_2_PBS_CHECKPOINT_SC_2_PBS_CHECKPOINT,
	LINUX_SC_V6_ILP32_OFF32_SC_V6_ILP32_OFF32,
	LINUX_SC_V6_ILP32_OFFBIG_SC_V6_ILP32_OFFBIG,
	LINUX_SC_V6_LP64_OFF64_SC_V6_LP64_OFF64,
	LINUX_SC_V6_LPBIG_OFFBIG_SC_V6_LPBIG_OFFBIG,
	LINUX_SC_HOST_NAME_MAX_SC_HOST_NAME_MAX,
	LINUX_SC_TRACE_SC_TRACE,
	LINUX_SC_TRACE_EVENT_FILTER_SC_TRACE_EVENT_FILTER,
	LINUX_SC_TRACE_INHERIT_SC_TRACE_INHERIT,
	LINUX_SC_TRACE_LOG_SC_TRACE_LOG
};


/* COMPAT: struct __sched_param */
struct linux__sched_param
{
	int				__sched_priority;
};

/* COMPAT: pthread_t
 * XXX: unsigned long int == struct pthread *
 */
typedef	unsigned long int		linux_pthread_t;

/* COMPAT: pthread_attr_t */
typedef	struct linux__pthread_attr_s
{
	int				__detachstate;
	int				__schedpolicy;
	struct linux__sched_param	__schedparam;
	int				__inheritsched;
	int				__scope;
	size_t				__guardsize;
	int				__stackaddr_set;
	void *				__stackaddr;
	size_t				__stacksize;
} linux_pthread_attr_t;

/* COMPAT: pthread_mutex */
typedef struct linux_pthread_mutex {
	int32_t dummy[6];
} linux_pthread_mutex_t;

/* COMPAT: pthread_mutexattr_t */
typedef struct linux_pthread_mutexattr {
	int32_t dummy;
} linux_pthread_mutexattr_t;


/* COMPAT: struct audio_buf_info */
#define linux_audio_buf_info                    audio_buf_info

/* COMPAT: struct count_info */
#define linux_count_info                        count_info

/* COMPAT: struct timespec */
struct linux_timespec {
        uint32_t                tv_sec;
        uint32_t                tv_nsec;
};

/* COMPAT: struct stat */
struct linux_stat {
        unsigned short int      st_dev;
        unsigned short int      __pad1;
        uint32_t                st_ino;
        unsigned short int      st_mode;
        unsigned short int      st_nlink;
        unsigned short int      st_uid;
        unsigned short int      st_gid;
        unsigned short int      st_rdev;
        unsigned short int      __pad2;
        uint32_t                st_size;
        uint32_t                st_blksize;
        uint32_t                st_blocks;
        struct linux_timespec   st_atimespec;
        struct linux_timespec   st_mtimespec;
        struct linux_timespec   st_ctimespec;
        uint32_t                __unused4;
        uint32_t                __unused5;
};

/*
typedef unsigned short int shmatt_t;
*/

#if 0
struct shmid_ds
  {
    struct ipc_perm shm_perm;           /* operation permission struct */
    int shm_segsz;                      /* size of segment in bytes */
    __time_t shm_atime;                 /* time of last shmat() */
    __time_t shm_dtime;                 /* time of last shmdt() */
    __time_t shm_ctime;                 /* time of last change by shmctl() */
    __pid_t shm_cpid;                   /* pid of creator */
    __pid_t shm_lpid;                   /* pid of last shmop */
    shmatt_t shm_nattch;                /* number of current attaches */
  };
#endif

/* COMPAT: struct utsname */
struct linux_utsname {
	char	sysname[UTSNAME_LENGTH];
	char	nodename[UTSNAME_LENGTH];
	char	release[UTSNAME_LENGTH];
	char	version[UTSNAME_LENGTH];
	char	machine[UTSNAME_LENGTH];
	char	domainname[UTSNAME_LENGTH];
};

/* COMPAT: sigset_t */
typedef	unsigned int	linux_sigset_t;

/* COMPAT: struct sigaction */
struct linux_sigaction {
	void (*lsa_handler)(int);
	linux_sigset_t	lsa_mask;
	unsigned int	lsa_flags;
	void (*lsa_restorer)(void);
} l_osigaction_t;


/* COMPAT: struct sockaddr */
struct linux_sockaddr {
	unsigned short	sa_family;
	char		sa_data[14];
};

#define	LINUX_SIGEMPTYSET(set)		(set) = 0
#define	LINUX_SIGISMEMBER(set, sig)	(set) & _SIG_BIT(sig)
#define	LINUX_SIGADDSET(set, sig)	(set) |= _SIG_BIT(sig)
